#!/usr/bin/env python
# Coolan Confidential
# Copyright (c) Coolan, Inc 2014 All Rights Reserved
#
# NOTICE:  All information contained herein is, and remains
# the property of Coolan, Inc.  The intellectual and
# technical concepts contained #herein are proprietary to
# Coolan, Inc and may be covered by U.S. and Foreign Patents,
# patents in process, and are protected by trade secret or copyright law.
# Dissemination of this information or reproduction of this material
# is strictly forbidden unless prior written permission is obtained
# from Coolan, Inc.
# This code and information are provided "as is" without warranty of any
# kind, either expressed or implied, including but not limited to the
# implied warranties of mechantability and/or fitness for a
# particular purpose.
import glob

from common import *
import settings as s


def run_commands(commands, log_path, via_shell=False):
    for full_cmd in commands:
        # lsmod command is hard coded
        if "lsmod" in full_cmd:
            run_lsmod_command(full_cmd, log_path)
        else:
            args = []
            args.extend(shlex.split(full_cmd))
            executable_cmd = args[0]
            executable_cmd_full_path = find_command(executable_cmd, full_cmd)
            if executable_cmd_full_path:
                # adding the full path of the command to the command we get from commands.properties
                args[0] = executable_cmd_full_path
                # if this is a cat command, copy the output to the log
                if executable_cmd == "cat":
                    if os.path.isfile(args[-1]):
                        f = open(args[-1], 'r')
                        write_command_output_to_log(f.read(), "", args, log_path)
                        f.close()
                        if "coolanMcelog" in args[-1]:
                            os.remove(args[-1])
                    else:
                        err = "File %s does not exist.\n" % args[-1]
                        write_command_output_to_log("", err, args, log_path)
                elif executable_cmd == "smartctl":
                    run_smartctl_command(args, log_path)
                elif executable_cmd in set(s.HPTOOL):
                    run_hp_raid_command(args, log_path)
                else:
                    command_out, command_err = get_command_output(args=args, via_shell=via_shell)
                    write_command_output_to_log(command_out, command_err, args, log_path)
            # this command does not exist on this machine
            else:
                err = "Command %s doesn't exist.\n" % args[0]
                write_command_output_to_log("", err, args, log_path)


def run_smartctl_command(smartctl_args, log_path):
    if smartctl_args[1] == '-V' or smartctl_args[1] == '--scan':
        smartctl_out, smartctl_err = get_command_output(args=smartctl_args)
        write_command_output_to_log(smartctl_out, smartctl_err, smartctl_args, log_path)
    elif smartctl_args[1] == '-x' or smartctl_args[1] == '-a':
        devices = []
        args_scan = [smartctl_args[0], '--scan']
        out, err = get_command_output(args=args_scan)
        for line in out.splitlines(True):
            devices.append(line.strip())

        for device in devices:
            smartctl_command_with_device = smartctl_args[:]
            device_command_args = shlex.split(device)
            device = device_command_args[0]
            device_type = device_command_args[2].split(',')[0]

            # this drive is behind a RAID controller
            # smartctl -x -d megaraid,N /dev/XXX
            if device_type in s.RAID_TYPES:
                smartctl_command_with_device.append("-d")
                smartctl_command_with_device.append(device_command_args[2])
                smartctl_command_with_device.append(device)

            # just a regular drive
            # smartctl -x /dev/XXX
            else:
                smartctl_command_with_device.append(device)

            out, err = get_command_output(args=smartctl_command_with_device)
            write_command_output_to_log(out, err, smartctl_command_with_device, log_path)

            # Run smartctl for the drives that don't show up in "smartctl --scan"
            # smartctl -x -d megaraid,1 /dev/sda
            if device_type not in s.RAID_TYPES:
                i = 0
                while True:
                    # smartctl -x -d megaraid,N /dev/XXX
                    smartctl_perc_raid_command = smartctl_args[0] + " -x -d megaraid,%s " + device
                    perc_smartctl_args = shlex.split(smartctl_perc_raid_command % i)
                    out, err = get_command_output(args=perc_smartctl_args)
                    if has_no_more_drives(out, err):
                        break
                    write_command_output_to_log(out, err, perc_smartctl_args, log_path)
                    i += 1


# For HP Smart Array RAID controllers
def run_hp_raid_command(hp_smartctl_args, log_path):
    if hp_smartctl_args[1] == 'version':
        out, err = get_hp_command_output(args=hp_smartctl_args)
        write_command_output_to_log(out, err, hp_smartctl_args, log_path)
    elif ' '.join(hp_smartctl_args[1:]) == "ctrl all show config detail":
        # do hpacucli rescan first
        rescan_args = shlex.split(hp_smartctl_args[0] + " rescan")
        get_command_output(args=rescan_args)
        out, err = get_hp_command_output(hp_smartctl_args)
        write_command_output_to_log(out, err, hp_smartctl_args, log_path)

        # now get the count of the drives behind hp smart array
        drive_count = out.lower().count("physicaldrive")

        # For disks accessed via the cciss driver the device nodes are of the form "/dev/cciss/c[0-9]d0".
        # For disks accessed via the hpahcisr and hpsa drivers, the device nodes you need are "/dev/sg[0-9]*".
        # Fpr FREEBSD, use "/dev/ciss[0-9]"
        split_pattern = get_split_pattern()
        hp_array_device_names = split_pattern.split(config.get(s.FILE_LOCATIONS, "hp_array_device_name").strip())
        for hp_array_device_name in hp_array_device_names:
            for possible_device_name in glob.glob(hp_array_device_name):
                smartctl_command_full_path = find_command("smartctl")

                # smartctl -x -A -d cciss,i /dev/sg0 OR smartctl -x -A -d cciss,i /dev/cciss/c0d0 OR smartctl -x -A -d cciss,i /dev/ciss0
                smartctl_hp_raid_command = smartctl_command_full_path + " -x -d cciss,%s " + possible_device_name
                for i in range(0, drive_count):
                    hp_smartctl_args = shlex.split(smartctl_hp_raid_command % i)
                    out, err = get_command_output(args=hp_smartctl_args)
                    if has_no_more_drives(out, err):
                        break
                    write_command_output_to_log(out, err, hp_smartctl_args, log_path)


# hpacucli can only have one instance running. collector will wait until the other instance is done.
def get_hp_command_output(args):
    timeout = int(config.get(s.SETTINGS, 'periodic_commands_time_out_seconds'))
    start = time.time()
    stop = False
    while not stop:
        out, err = get_command_output(args=args)
        for hp_err_msg in s.HPTOOL_ERROR_OUTPUT:
            if hp_err_msg.lower() not in out.lower() and hp_err_msg.lower() not in err.lower():
                return out, err
        time.sleep(1)
        stop = time.time() - start > timeout
    return out, err


def has_no_more_drives(out, err):
    for smart_err_msg in s.SMARTTOOL_ERROR_OUTPUT:
        if smart_err_msg.lower() in err.lower() or smart_err_msg.lower() in out.lower():
            return True
    return False


def run_lsmod_command(lsmod_command, log_path):
    """
    Hard-coded for lsmod
    # for i in ` lsmod | awk '{print $1}' `;do modinfo $i | grep -v alias | grep -v firmware; done
    """

    command_indices = [4, 6, 10, 13]
    args = lsmod_command.split(" ")
    bash_commands = [args[i] for i in command_indices]

    # add the nice wrapper and the command path to the bash commands
    for index, bash_command in enumerate(bash_commands):
        command_full_path = find_command(bash_command)
        if command_full_path:
            lsmod_command = lsmod_command.replace(bash_command, global_nice_wrapper + " " + command_full_path)
        else:
            index -= 1
            break

    # run the lsmod command
    if index + 1 == len(command_indices):
        out, err = get_command_output(args=lsmod_command.split(" "), via_shell=True, add_nice_wrapper=False)
        write_command_output_to_log(out, err, lsmod_command.split(" "), log_path)
    # there is a bash command in the lsmod command that does not exist on this machine
    else:
        err = "Command %s doesn't exist.\n%" % bash_commands[index + 1]
        write_command_output_to_log("", err, args, log_path)


def main(test):
    try:
        verify_root()
        print "%s : Started collecting data." % get_timestamp()

        start_time = time.time()

        last_run_daily = get_last_run_daily_time()
        new_day = (start_time - last_run_daily) >= 24 * 60 * 60

        last_run = get_last_run_time()

        old_boot_id = get_boot_id_setting()
        new_boot_id = get_new_boot_id()
        rebooted = old_boot_id != new_boot_id

        old_machine_id = get_machine_id_setting()
        new_machine_id = get_new_machine_id()

        set_commands_path()


        # Get all the periodic commands
        commands_set = get_commands(s.PERIODIC_COMMANDS)
        commands_set = commands_set.union(get_commands(s.COMMON_PERIODIC_COMMANDS))

        # if it's the first time running the collector in non-testing mode
        # or the machine was rebooted
        # or it's in testing mode
        if not last_run or rebooted or test:
            # Add boot commands
            commands_set = commands_set.union(get_commands(s.BOOT_COMMANDS))
            # Add daily commands
            commands_set = commands_set.union(get_commands(s.DAILY_COMMANDS))
            add_property_to_settings('LAST_RUN_DAILY', start_time)
        # if it's a new day
        if new_day:
            # Add daily commands
            commands_set = commands_set.union(get_commands(s.DAILY_COMMANDS))
            add_property_to_settings('LAST_RUN_DAILY', start_time)
        if is_config_on('run_vm_commands'):
            # Add VM commands
            commands_set = commands_set.union(get_commands(s.VM_COMMANDS))
        if is_config_on('run_hp_commands'):
            # Add HP commands
            commands_set = commands_set.union(get_commands(s.HP_SMART_ARRAY_COMMANDS))
        # add the last run time if it's running in the non-test mode
        if not test:
            add_property_to_settings('LAST_RUN', start_time)

        # Finishing setting up. Begin collecting data.
        log_path = "%s.log%s" % (os.path.join(get_logs_folder_path(), generate_log_filename()), s.TMP_FILE_EXTENSION)
        run_commands(commands_set, log_path)
        commands_set = get_commands(s.PERIODIC_VIA_SHELL_COMMANDS)
        run_commands(commands_set, log_path, via_shell=True)

        # Print the general information section
        log_file = open(log_path, 'a')
        log_file.write(s.BEGIN_MSG % "General Information")
        log_file.write("testMode : %s\n" % test)
        log_file.write("collectorVersion : %s\n" % s.COLLECTOR_VERSION)
        log_file.write("pythonVersion : %s\n" % sys.version)
        log_file.write("collectorStartTimeSeconds : %s\n" % int(start_time))
        end_time = time.time()
        log_file.write("collectorEndTimeSeconds : %s\n" % int(end_time))
        log_file.write("runningTimeSeconds : %s\n" % str(end_time - start_time))
        log_file.write("previousMachineId : %s\n" % old_machine_id)
        log_file.write("currentMachineId : %s\n" % new_machine_id)
        log_file.write("previousBootId : %s\n" % old_boot_id)
        log_file.write("currentBootId : %s\n" % new_boot_id)
        log_file.write("runDailyCommands : %s\n" % (new_day or rebooted or test or not last_run))
        log_file.write("runBootCommands : %s\n" % (rebooted or test or not last_run))
        print_dmidecode_info(log_file)
        log_file.write(s.END_MSG % "General Information")
        log_file.close()

        # Rename the log file to remove the tmp file extension
        new_log_full_path = log_path.replace(s.TMP_FILE_EXTENSION, "")
        os.rename(log_path, new_log_full_path)
        print "%s : Finished collecting data. %s has been created. " % (get_timestamp(), new_log_full_path)
    except:
        print_fail("%s : There are errors when generating the logs." % get_timestamp())
        exc_type, exc_value, exc_traceback = sys.exc_info()
        traceback.print_exception(exc_type, exc_value, exc_traceback, limit=2, file=sys.stdout)
        os._exit(1)
    finally:
        sys.stdout.flush()


if __name__ == "__main__":
    main(test=True)
