#!/usr/bin/env python
# Coolan Confidential
# Copyright (c) Coolan, Inc 2014 All Rights Reserved
#
# NOTICE:  All information contained herein is, and remains
# the property of Coolan, Inc.  The intellectual and
# technical concepts contained #herein are proprietary to
# Coolan, Inc and may be covered by U.S. and Foreign Patents,
# patents in process, and are protected by trade secret or copyright law.
# Dissemination of this information or reproduction of this material
# is strictly forbidden unless prior written permission is obtained
# from Coolan, Inc.
# This code and information are provided "as is" without warranty of any
# kind, either expressed or implied, including but not limited to the
# implied warranties of mechantability and/or fitness for a
# particular purpose.

import coolanRunner
from common import *


def verify():
    """
    Run this function to set up the environment. It's designed to be run only at the first time running the script.
    """
    print "%s : Start verifying Coolan configuration." % get_timestamp()
    # Verify that the script is being run as root, bail otherwise.
    verify_root()

    # Create a verify log file to record the verify output
    tmp_verify_log_path = os.path.join(get_logs_folder_path(), 'verify.log') + s.TMP_FILE_EXTENSION
    f = open(tmp_verify_log_path, 'w')
    f.write("Script Path: " + script_path + "\n")

    verify_auth_section()

    # do not save the machine id when doing verify config
    get_new_machine_id()

    # do not save the boot id when doing verify config
    get_new_boot_id()

    set_commands_path()

    # Get all the commands
    all_commands_and_description_map = get_all_commands_and_description()

    # Print out all missing commands
    missing_commands = {}
    verified_commands = {}
    for cmdItem in all_commands_and_description_map.items():
        cmd = cmdItem[0]
        args = []
        args.extend(shlex.split(cmd))

        # hard-coded for lsmod
        # for i in ` lsmod | awk '{print $1}' `;do modinfo $i | grep -v alias | grep -v firmware; done
        if "lsmod" in cmd and verified_commands.get("lsmod") is not True:
            args = cmd.split(" ")
            sub_commands = [args[i] for i in [4, 6, 10, 13]]

            for index, oldSubCmd in enumerate(sub_commands):
                verified_commands[oldSubCmd] = True
                command_full_path = find_command(oldSubCmd)
                if not command_full_path:
                    missing_commands[oldSubCmd] = cmdItem[1]
        elif verified_commands.get(args[0]) is not True:
            # searching for the command on the machine
            verified_commands[args[0]] = True
            command_full_path = find_command(args[0], cmd)
            # if the command is not found and it's not in the optional commands list, add it to the missing commands set
            if not command_full_path and args[0] not in s.OPTIONAL_COMAANDS:
                missing_commands[args[0]] = cmdItem[1]
    if missing_commands:
        print_fail("WARN: The following commands are missing and will not be run.")
        f.write("WARN: The following commands are missing and will not be run.\n")
        for key, value in dict.items(missing_commands):
            print key + " : " + value
            f.write(key + " : " + value + "\n")

    print ""

    https_proxy = get_proxy_url()
    if https_proxy:
        f.write("Using Proxy:" + https_proxy + "\n")
    else:
        f.write("Not using proxy.\n")

    f.close()
    verify_log_filename = generate_verify_log_filename()
    os.rename(tmp_verify_log_path, os.path.join(get_logs_folder_path(), verify_log_filename))

    is_post_success = False
    if is_config_on('send_logs_to_endpoint'):
        # test POSTing zip
        is_post_success = test_posting(verify_log_filename)
    else:
        print "%s : Sending log to Coolan endpoint has been turned off. Skip testing sending logs." % get_timestamp()

    print "%s : Done verifying Coolan configuration." % get_timestamp()

    if is_post_success or not is_config_on('send_logs_to_endpoint'):
        coolanRunner.run(test=True)


def test_posting(file_name):
    file_path = os.path.join(get_logs_folder_path(), file_name)

    url = get_endpoint() + s.LOGS_ENDPOINT_PATH % (get_customer_id(), file_name)
    is_post_success = False
    try:
        is_post_success = post_file(url, get_username(), get_password(), file_path)
    except:
        print_fail('\nUnable to make connections to Coolan endpoint:')
        print_fail('  1. Please make sure the "proxyUrl" is being set correctly if you are using proxy.')
        print_fail('  2. Please make sure your machine is able to POST to Coolan endpoint: ' + get_endpoint())
        print_fail('  3. Please verify that your openssl is up to date.')
    finally:
        # remove the verify log whether the post was successful or not
        os.remove(file_path)

    return is_post_success


if __name__ == "__main__":
    verify()
